@@ -1,7 +1,7 @@
 // { dg-options "-pedantic -std=gnu++0x" }
 // { dg-do compile }
 
-// Copyright (C) 2011-2013 Free Software Foundation, Inc.
+// Copyright (C) 2011-2014 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -22,13 +22,7 @@
 
 // PR libstdc++/49058
 
-struct F
-{
-  void
-  operator()();
-};
-
 void f()
 {
-  std::bind( F() );
+  std::bind( []{} );
 }
