@@ -1,6 +1,4 @@
-// { dg-options "-std=gnu++0x" }
-
-// Copyright (C) 2009-2013 Free Software Foundation, Inc.
+// Copyright (C) 2009-2014 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -27,37 +25,32 @@ void test01()
   using namespace std;
 
   valarray<int> v1;
+  const valarray<int> v2(-1, 10000);
 
-  v1 = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };
-  VERIFY( v1.size() == 10 );
-  VERIFY( v1.min() == -1 );
-  VERIFY( v1.max() == -1 );
+  v1 = v2;
+  VERIFY( v1.size() == v2.size() );
+  VERIFY( (v1 == v2).min() == true );
 
-  valarray<int> v2(0, 10);
+  valarray<int> v3(0, 10000);
+  const valarray<int> v4;
 
-  v2 = { };
-  VERIFY( v2.size() == 0 );
+  v3 = v4;
+  VERIFY( v3.size() == v4.size() );
+  VERIFY( v3.size() == 0 );
 
-  valarray<int> v3(0, 10);
+  valarray<int> v5(0, 100);
+  const valarray<int> v6(-1, 10000);
 
-  v3 = { -1, -1, -1, -1, -1 };
-  VERIFY( v3.size() == 5 );
-  VERIFY( v3.min() == -1 );
-  VERIFY( v3.max() == -1 );
+  v5 = v6;
+  VERIFY( v5.size() == v6.size() );
+  VERIFY( (v5 == v6).min() == true );
 
-  valarray<int> v4(0, 5);
+  valarray<int> v7(0, 10000);
+  const valarray<int> v8(-1, 100);
 
-  v4 = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };
-  VERIFY( v4.size() == 10 );
-  VERIFY( v4.min() == -1 );
-  VERIFY( v4.max() == -1 );
-
-  valarray<int> v5(0, 10);
-
-  v5 = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };
-  VERIFY( v5.size() == 10 );
-  VERIFY( v5.min() == -1 );
-  VERIFY( v5.max() == -1 );
+  v7 = v8;
+  VERIFY( v7.size() == v8.size() );
+  VERIFY( (v7 == v8).min() == true );
 }
 
 int main()
