@@ -1,10 +1,10 @@
-// { dg-do run { xfail *-*-* } }
+// { dg-do run }
 // { dg-options "-std=c++0x" }
 
 //
 // 2010-06-23  Stephen M. Webb <stephen.webb@bregmasoft.ca>
 //
-// Copyright (C) 2010 Free Software Foundation, Inc.
+// Copyright (C) 2010-2014 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -35,10 +35,28 @@ test01()
   typedef char CharT;
   typedef std::regex_traits<CharT> traits;
 
-	char name[] = "lower";
-	traits t;
+  const CharT lower[]   = "lOWer";
+  const CharT upper[]   = "UPPER";
+  const CharT nothing[] = "nothing";
+  const CharT word[]    = "w";
+  const CharT blank[]   = "blank";
+  const CharT digit[]   = "digit";
+  traits t;
 
-	VERIFY( t.isctype('e',  t.lookup_classname(name, name+sizeof(name)-1)) );
+#define range(s) s, s+sizeof(s)/sizeof(s[0])-1
+  VERIFY( t.isctype('_', t.lookup_classname(range(word))));
+  VERIFY( t.isctype('A', t.lookup_classname(range(word))));
+  VERIFY(!t.isctype('~', t.lookup_classname(range(word))));
+  VERIFY(!t.isctype('e', t.lookup_classname(range(upper))));
+  VERIFY( t.isctype('e', t.lookup_classname(range(lower))));
+  VERIFY(!t.isctype('e', t.lookup_classname(range(nothing))));
+  VERIFY(!t.isctype('_', t.lookup_classname(range(digit))));
+  VERIFY( t.isctype(' ', t.lookup_classname(range(blank))));
+  VERIFY( t.isctype('\t', t.lookup_classname(range(blank))));
+  VERIFY(!t.isctype('\n', t.lookup_classname(range(blank))));
+  VERIFY( t.isctype('t', t.lookup_classname(range(upper), true)));
+  VERIFY( t.isctype('T', t.lookup_classname(range(lower), true)));
+#undef range
 }
 
 int main()
